#install.packages("plotrix")
#install.packages("diagram")
library("plotrix")
library("diagram")
# --------------------------------------------------------------------------------------------------
# 
# (c) Laurent Excoffier, March-April 2020
# Modified by Nina Marchi June 2020
# Analysis of the results of fastsimcoal
# 
# The script will, for each studied model
# 1) Extract the best run among those that have been performed
# 2) Draw the ML scenario for each model
# 3) Print the marginal lhoods for each population under each model
# 4) Print the worst fitted SFS entries 
# 
# the script requires ParFileInterpreter-v6.3.5.r, which should be in the same directory as this one
# 
# --------------------------------------------------------------------------------------------------

#Specify how many runs were done per model (here same number is assumed for all models)
numRuns=50 ##TO BE MODIFIED

#--- Define population labels that are used in different plots
##ONLY LIST SAMPLED POPULATIONS
popLabels.Test=c("POP1","POP2","POP3","POP4","POP5","POP6")  ##TO BE MODIFIED

#--- Location of the root directory (where the R script should be) a network directory should work
baseDir=""  ##TO BE MODIFIED

#--- Define which models to analyse here ---------------------
curModels=c("Model1","Model2")  ##TO BE MODIFIED

#--- Define where these models are located relative to baseDir
#The results of the different runs should be in those directories
#The name of the directory is assumed to be the name of the model followed by /run1, /run2, etc...
curDirs=rep("Scenario1",2)  ##TO BE MODIFIED

#--- Specify which pop labels are valid for the different models 
pop.Labels=vector("list", length=length(curModels))  ##TO BE MODIFIED
pop.Labels[[1]]=popLabels.Test
pop.Labels[[2]]=popLabels.Test

#--- Define name of the result summary file here ---------------------
# these files will contain the max Lhood params of each model as well as the max obs and est logLhoods
result.file=paste(baseDir,"Test.txt", sep="")
result.plot=paste(baseDir,"Test", sep="")
MatLhoods=c()

doEverything=F
extractBestRun        =F
drawModel             =F
drawMarginalSFS       =F
printPDF              =F
printPNG              =F
summarizeRresults     =F
plotWorstFitSFS       =F

if (doEverything) {
   extractBestRun        =T
   drawModel             =T
   drawMarginalSFS       =T
   printPDF              =T
   printPNG              =T
   summarizeRresults     =T
   plotWorstFitSFS       =T
} else {  #Select what to do
   extractBestRun        =T
   drawModel             =F
   drawMarginalSFS       =T
   printPDF              =T
   printPNG              =T
   summarizeRresults     =T
   plotWorstFitSFS       =T
}

numBadEntriesToPLot=30 #For worse fitted SFS entries in each model

### get entries labels -----------------------------------------------------------------------------
# takes a vector of pop size and returns entries labels
# Alexandre Gouy - 09-2017
getEntriesLabels <- function(pop.sizes) {
   if(prod(pop.sizes) >= 1e6) {
      stop("Too many entries (> 1e6).")
   }
   inv.pop.sizes=pop.sizes[length(pop.sizes):1]
   li <- lapply(inv.pop.sizes, seq, from = 0, by = 1)
   entr <- expand.grid(li)
   entr <- rev(entr)
   entries.lab <- apply(entr, 1, function(x) {
      paste0("(", paste(x, collapse = ","), ")")
   })
   return(entries.lab)
}

#---------------------------------------------------------------------------------------------------
plot.marginal1D.sfs=function(obs.sfs, exp.sfs, mar, title, log) { 
   cols=c("orange", "cornflowerblue")
   #Remove observed monomorphic sites
   obs.sfs[1]=0
   #Get marginal SFS
   obs=apply(obs.sfs, mar, sum)
   exp=apply(exp.sfs, mar, sum)
   tot.num.sites=sum(obs)
   exp.abs=exp*tot.num.sites
   
   miny=min(c(obs, exp.abs))
   maxy=max(c(obs, exp.abs))
   if(log) logy="y" else logy=""
   plot(x=0:(length(obs)-1),obs, type="b", col=cols[1], lwd=2, xlab="i", ylab="sfs[i]", pch=16, cex=1.2, 
        main = title, log=logy, ylim=c(miny, maxy))
   lines(x=0:(length(obs)-1), exp.abs, col=cols[2], lwd=2, type="b" , pch=16, cex=1.2)
   legend("topright", legend=c("observed", "expected"), col=cols, lwd=2)
}

#---------------------------------------------------------------------------------------------------
#---------------------------------------------------------------------------------------------------
#                                      BEGINNING OF THE MAIN LOOP
#---------------------------------------------------------------------------------------------------
#---------------------------------------------------------------------------------------------------

bestEstLhoods=1:length(curModels)*0
bestObsLhoods=1:length(curModels)*0
bestParams=vector("list", length=length(curModels))
   
curcase=0

for (genericName in curModels) {
   curcase=curcase +1 
   curDir=paste(baseDir,curDirs[curcase], sep="")
   setwd(curDir)
   cat("----------------------------------------------------------------------------------------\n")
   cat(paste("Working on model ", genericName, "\n in directory ", curDir, "\n", sep=""))
   cat("----------------------------------------------------------------------------------------\n")
   
   popLabels=pop.Labels[[curcase]]
   #===================================================================================================
   # Find which run has best lhood-------------------------------------------------------------------
   #===================================================================================================
   if (extractBestRun) {
      cat("Extracting best lhood files\n")
      MatLhood=c()
      MatVal=c()
      maxLhood=-10^100
      for (i in 1:numRuns) {
         print(i)
         localDir=paste(curDir, "/", genericName, "/run", i, sep="")
         curbestFile=paste(localDir, "/", genericName,"/", genericName,".bestlhoods", sep="")
         curBestParams=read.table(curbestFile, header=T)
         MatVal=rbind(MatVal,curBestParams)
         curMaxLhood = curBestParams[["MaxEstLhood"]]
         MatLhood=c(MatLhood,curMaxLhood)
         if (curMaxLhood>maxLhood) {
            maxLhood=curMaxLhood
            bestRun=i
            bestDir=curDir
         }
      }
      bestRun
      maxLhood
      MatLhoods=cbind(MatLhoods,MatLhood)
      write.table(MatVal, file=paste(curDir,"/",genericName,".txt", sep=""), quote=F, sep="\t", col.names = T, row.names = F)
      
      # Copy best run into a specific directory + PLOT --------------------------------------------------------
      bestDir=paste(genericName, "-bestRun-OO", numRuns, sep="")
      if (dir.exists(bestDir)) { #remove directory and all of its content before copying new files
         
      }
      cmd=paste("cp -r ", genericName, "/run", bestRun, "/. ", bestDir, sep="")
      system(cmd)
      cat("done!\n\n")
   }
   
   #===================================================================================================
   # Plot lhood optimisation-------------------------------------------------------------------
   #===================================================================================================
   if (extractBestRun) {
     cat("Extracting and plotting lhood\n")
     
     MatBrent=c()
     for (i in 1:numRuns) {
       localDir=paste(curDir, "/", genericName, "/run", i, sep="") #MODIF /run and not -run
       curFile=paste(localDir, "/", genericName,"/", genericName,".brent_lhoods", sep="") #MODIF Another folder into run/
       BRENT=read.delim(curFile)
       BRENT=BRENT[-nrow(BRENT),]
       MatBrent=cbind(MatBrent,BRENT[,"MaxEstLhood"])
     }
     
     if (printPDF) {
       pdfFileName=paste(curDir, "/",genericName,"_MaxEstLhoodCurve.pdf", sep="")
       cat(paste("   Plotting likelihoods to file", pdfFileName), sep="\n")
       pdf(pdfFileName, height=10, width=8)
       plot(seq(1,nrow(Mat),1),Mat[,1], ylim=c(1.002*max(Mat),max(Mat)), type='l', lwd=1, xlab="Update",ylab="", cex.main=0.7,main=paste("MaxEstLhood for ",genericName,sep=""), las=2)
       for (i in c(2:numRuns)){
         lines(seq(1,nrow(Mat),1),Mat[,i], type='l', lwd=1)
       }
       #best run brent_lhoods
       bestDir=paste(genericName, "-bestRun-OO", numRuns, sep="")
       bestFile=paste(bestDir, "/", genericName,"/", genericName,".brent_lhoods", sep="") #MODIF Another folder into run/
       BEST=read.delim(bestFile)
       BEST=BEST[-nrow(BEST),]
       lines(seq(1,nrow(BEST),1),BEST[,"MaxEstLhood"], type='l', lwd=1, col="red")
       legend("bottomright",lty=1, col="red",legend=paste("Best run: ",bestRun,sep=""), inset=0.01, box.col = NA)
       dev.off()
     }
     
     if (printPNG) {
       pngFileName=paste(curDir, "/",genericName,"_MaxEstLhoodCurve.png", sep="")
       cat(paste("   Plotting likelihoods to file", pngFileName, "\n"))
       png(pngFileName, height=3000, width=2000, res=300)
       par(mar=c(5,5,2,1))
       plot(seq(1,nrow(MatBrent),1),MatBrent[,1], ylim=c(1.002*max(MatBrent),max(MatBrent)), type='l', lwd=1, xlab="Update",ylab="", cex.main=0.7,main=paste("MaxEstLhood for ",genericName,sep=""), las=2)
       if (numRuns>1){
         for (i in c(2:numRuns)){
            lines(seq(1,nrow(MatBrent),1),Mat[,i], type='l', lwd=1)
         }
       }
       #best run brent_lhoods
       bestDir=paste(genericName, "-bestRun-OO", numRuns, sep="")
       bestFile=paste(bestDir, "/", genericName,"/", genericName,".brent_lhoods", sep="")
       BEST=read.delim(bestFile)
       BEST=BEST[-nrow(BEST),]
       lines(seq(1,nrow(BEST),1),BEST[,"MaxEstLhood"], type='l', lwd=1, col="red")
       legend("bottomright",lty=1, col="red",legend=paste("Best run: ",bestRun,sep=""), inset=0.01, box.col = NA)
       dev.off()
     }
     cat("done!\n\n")
   }
   #===================================================================================================
   # Draw best scenario -------------------------------------------------------------------------------
   #===================================================================================================

   if (drawModel) {
      
      #Extract maxLhood values
      bestDir=paste(genericName, "-bestRun-OO", numRuns, sep="")
      
      curbestFile=paste(bestDir, "/", genericName,"/", genericName,".bestlhoods", sep="")
      curBestParams=read.table(curbestFile, header=T)
      maxEstLhood = curBestParams[["MaxEstLhood"]]
      maxObsLhood = curBestParams[["MaxObsLhood"]]
      
      
      cat(paste("Printing sketch of demographic scenario.....\n")) 
      
      require(diagram, quietly  = T)
      
      #Extract mutation rate parameter from max_L.par file ------------------
      parFileName=paste(bestDir,"/", genericName, "/", genericName, "_maxL.par", sep="")
      parFile=readLines(parFileName)
      mut.rate.line.number=grep(pattern = "FREQ", x=parFile)
      mut.rate.line=parFile[mut.rate.line.number]
      #Mut rate should be 4th element
      curMutRate=as.numeric(unlist(strsplit(mut.rate.line, " "))[4])
      
      
      MaxNumGen=500
      fsc.rescaling.factor=1
      
      genTime=29                   
      realMutRate=curMutRate         
      fixMaxTime=T
      maxTimeToPlot=genTime*MaxNumGen   #Upper limit for time scale if fixMaxTime==T
      mutRateRescaling=1 #relative mutation rate rescaling
      rescalingFactor=1
      
      usePopLabels=T
      
      useExternalValues=T #Option to use the parameter values defined above in parFileInterpreter
      source("../ParFileInterpreter-v6.3.5.r")
      
      cat("done!\n\n")
   }
   
   #===================================================================================================
   #-----------  Comparing observed and expected SFS  ----------------------------------------------
   #===================================================================================================
   if (drawMarginalSFS)  {
      
      cat("Drawing marginal SFS ...\n")
      #Extract maxLhood values
      bestDir=paste(genericName, "-bestRun-OO", numRuns, sep="")
      
      curbestFile=paste(bestDir, "/", genericName,"/", genericName,".bestlhoods", sep="")  #MODIF Another folder into run/
      curBestParams=read.table(curbestFile, header=T)
      maxEstLhood = curBestParams[["MaxEstLhood"]]
      maxObsLhood = curBestParams[["MaxObsLhood"]]
      
      cat(paste("Printing sketch of demographic scenario.....\n")) 
      
      sfsFile.obs=paste(genericName,"_DSFS.obs", sep="")
      sfsFile.exp=paste(bestDir, "/" , genericName,"/" , genericName, "_DSFS.txt", sep="")  #MODIF Another folder into run/
      
      #---- Reading number of dimensions and sample sizes ---------------
      dim.sfs=read.table(file=sfsFile.obs, skip=1, header=F,nrows=1)
      numDims=as.numeric(dim.sfs[1])
      dim.sfs=dim.sfs[-1]
      #Inverting number of dimension to reflect the difference in SFS ordering and normal R matrix ordering
      dim.sfs=as.numeric(dim.sfs+1, nrow=1)[numDims:1]
      
      #---- Reading obs and exp multidimensional sfs ---------------
      sfs.obs=as.matrix(read.table(file=sfsFile.obs, skip=2, header=F, nrows=1))
      #Extract number of dimensions
      sfs.obs=array(data=sfs.obs, dim=dim.sfs)
      sfs.obs[1]=0
      # Transposition of the matrix entries
      sfs.obs=aperm(sfs.obs, numDims:1)
      
      sfs.exp=as.matrix(read.table(file=sfsFile.exp, skip=2, header=F, nrows=1))
      #Extract number of dimensions
      sfs.exp=array(data=sfs.exp, dim=dim.sfs)
      # Transposition of the matrix entries
      sfs.exp=aperm(sfs.exp, numDims:1)
      
      if (printPDF) {
         pdfFileName=paste(bestDir, "/", genericName,"_sfs_fit.pdf", sep="")
         cat(paste("   Printing obs and exp sfs to file", pdfFileName), sep="\n")
         pdf(pdfFileName, height=10, width=8)
         mat.layout=rbind(c(1,1), c(2,3), c(4,5), c(6,7))
         layout(mat.layout, height=c(0.15,1,1,1))
         par(mar=c(1,1,1,1))
         plot.new()
         mtext(paste(genericName,"_sfs_fit\n","EstLhood=", maxEstLhood, "   ObsLhood=", maxObsLhood, sep=""), side=1, cex=0.8)
         par(mar=c(4,4,3,1))
         for (i in 1:numDims) {
            plot.marginal1D.sfs(obs.sfs=sfs.obs, exp.sfs=sfs.exp, mar=i, pop.Labels[[curcase]][i], log=F)
         }
         if (numDims<6) for (p in 1:(6-numDims)) plot.new() else
            if (numDims>6) for (p in 1:(numDims%%6))  plot.new()
         layout(mat.layout, height=c(0.15,1,1,1))
         par(mar=c(1,0,0,0))
         plot.new()
         mtext(paste(genericName,"_sfs_fit\n","EstLhood=", maxEstLhood, "   ObsLhood=", maxObsLhood, sep=""), side=1, cex=0.8)
         par(mar=c(4,4,2,1))
         for (i in 1:numDims) {
            plot.marginal1D.sfs(obs.sfs=sfs.obs, exp.sfs=sfs.exp, mar=i, pop.Labels[[curcase]][i], log=T)
         }
         dev.off()
      }
      
      if (printPNG) {
         pngFileName=paste(bestDir, "/", genericName,"_sfs_fit_abs_scale.png", sep="")
         cat(paste("   Printing obs and exp sfs to file", pngFileName, "\n"))
         mat.layout=rbind(c(1,1), c(2,3), c(4,5), c(6,7))
         png(pngFileName, height=3000, width=2000, res=300)
         layout(mat.layout, height=c(0.15,1,1,1))
         par(mar=c(1,0,0,0))
         plot.new()
         mtext(paste(genericName,"_sfs_fit\n","EstLhood=", maxEstLhood, "   ObsLhood=", maxObsLhood, sep=""), side=1, cex=0.8)
         par(mar=c(4,4,2,1))
         for (i in 1:numDims) {
            plot.marginal1D.sfs(obs.sfs=sfs.obs, exp.sfs=sfs.exp, mar=i, pop.Labels[[curcase]][i], log=F)
         }
         dev.off()
         
         pngFileName=paste(bestDir, "/", genericName,"_sfs_fit_log_scale.png", sep="")
         png(pngFileName, height=3000, width=2000, res=300)
         layout(mat.layout, height=c(0.15,1,1,1))
         par(mar=c(1,0,0,0))
         plot.new()
         mtext(paste(genericName,"_sfs_fit\n","EstLhood=", maxEstLhood, "   ObsLhood=", maxObsLhood, sep=""), side=1, cex=0.8)
         par(mar=c(4,4,2,1))
         for (i in 1:numDims) {
            plot.marginal1D.sfs(obs.sfs=sfs.obs, exp.sfs=sfs.exp, mar=i, pop.Labels[[curcase]][i], log=T)
         }
         dev.off()
      }
      cat("done!\n", sep="\n")
   }
   
   
   #===================================================================================================
   # Plot worst fitted entries in the SFS -------------------------------------------------------------
   #===================================================================================================
   sfsFile.obs=paste(genericName,"_DSFS.obs", sep="")
   sfsFile.exp=paste(bestDir, "/" , genericName,"/" , genericName, "_DSFS.txt", sep="")
   
   #---- Reading number of dimensions and sample sizes ---------------
   dim.sfs=read.table(file=sfsFile.obs, skip=1, header=F,nrows=1)
   numDims=as.numeric(dim.sfs[1])
   dim.sfs=dim.sfs[-1]
   #Inverting number of dimension to reflect the difference in SFS ordering and normal R matrix ordering
   dim.sfs=as.numeric(dim.sfs+1, nrow=1)[numDims:1]
   
   
   #---- Reading obs and exp multidimensional sfs ---------------
   sfs.obs=as.matrix(read.table(file=sfsFile.obs, skip=2, header=F, nrows=1))
   if (plotWorstFitSFS) {
      cat(paste("Plotting worse fitted SFS entries", curcase), sep="\n")
      #Read obs and exp SFS
      #Extract maxLhood values
      bestDir=paste(genericName, "-bestRun-OO", numRuns, sep="")
      cat("   Reading obs and exp SFS ...\n")
      sfsFile.obs=paste(genericName,"_DSFS.obs", sep="")
      sfsFile.exp=paste(bestDir,"/" , genericName, "/" , genericName, "_DSFS.txt", sep="")   #MODIF Another folder into run/
      
      #---- Reading obs multidimensional sfs ---------------
      sfs.obs=as.matrix(read.table(file=sfsFile.obs, skip=2, header=F, nrows=1))
      
      #---  Extract number of polymorphic sites
      mono.pos=c(1,length(sfs.obs))
      sfs.obs.pol=sfs.obs[-mono.pos]
      num.polym.sites=sum(sfs.obs.pol)
      
      #---- Reading exp multidimensional sfs ---------------
      sfs.exp=as.matrix(read.table(file=sfsFile.exp, skip=2, header=F, nrows=1))
      penalty=1e-10
      #replace all zero entries in exp sfs by a small arbitrary value
      sfs.exp[sfs.exp==0]=penalty
      #--- Getting expected SFS in absolute numbers
      sfs.exp.pol=sfs.exp[-mono.pos]*num.polym.sites
      cat("   done!\n")
      
      #--- Computing logLhood individual entries
      lhoods.obs=sfs.obs.pol*log10(sfs.exp[-mono.pos])
      lhood.obs.total=sum(lhoods.obs)
      
      #--- computing max possible lhoods
      sfs.exp.max=sfs.obs.pol/num.polym.sites
      #Set zero obs entries to 1
      sfs.exp.max[sfs.obs.pol==0]=1
      
      lhoods.exp.max=sfs.obs.pol*log10(sfs.exp.max)
      lhoods.exp.max.total=sum(lhoods.exp.max)
      
      #--- Computing difference in lhoods and order by decreasing difference
      dif.lhoods=lhoods.exp.max-lhoods.obs
      ordered.indices=order(abs(dif.lhoods),decreasing =T)
      
      #--- extract SFS entries labels
      popSizes=read.table(file=sfsFile.obs, skip=1, header=F, nrows=1)[-1]
      sfs.labels=getEntriesLabels(popSizes)[-mono.pos]
      
      
      #--- Plot worse SFS entries
      numBadEntriesToPLotBIS=min(length(sfs.labels[ordered.indices]),numBadEntriesToPLot)
      worse.sfs.labels=sfs.labels[ordered.indices][1:numBadEntriesToPLotBIS]

      
      if (printPDF) {
         pdfFileName=paste(bestDir, "/", genericName,"_",numBadEntriesToPLotBIS, "_worse_sfs_entries.pdf", sep="")
         cat(paste("   Plotting worse fitted sfs entries to file", pdfFileName, "\n"))
         pdf(pdfFileName, height=10, width=8)
         par(mfrow=c(2,1))
         par(mar=c(7,4,2,1))
         barplot(dif.lhoods[ordered.indices][1:numBadEntriesToPLotBIS], 
                 names.arg = worse.sfs.labels,las=2,
                 ylab="logLhood difference (max-est logLhoods)", main=paste(numBadEntriesToPLotBIS, "worse fitted SFS entries"), 
                 cex.names=0.8)
         barplot(rbind(sfs.obs.pol,sfs.exp.pol)[,ordered.indices][,1:numBadEntriesToPLotBIS], col=c("black","grey"), 
                 names.arg = worse.sfs.labels, beside = TRUE, las=2, ylab= "number of sites", cex.lab=1, 
                 main=paste(numBadEntriesToPLotBIS, "worse fitted SFS entries"), 
                 cex.names=0.8)
         legend("top", legend =c("Observed", "Expected"), pch=15,  col=c("black","grey"), cex=0.8)
         par(mfrow=c(1,1))
         dev.off()
      }
      if (printPNG) {
         pngFileName=paste(bestDir, "/", genericName,"_",numBadEntriesToPLotBIS, "_worse_sfs_entries.png", sep="")
         cat(paste("   Plotting worse fitted sfs entries to file", pngFileName, "\n"))
         png(pngFileName, height=3000, width=2000, res=300)
         par(mfrow=c(2,1))
         par(mar=c(7,4,2,1))
         barplot(dif.lhoods[ordered.indices][1:numBadEntriesToPLotBIS], 
                 names.arg = worse.sfs.labels,las=2,
                 ylab="logLhood difference (max-est logLhoods)", main=paste(numBadEntriesToPLotBIS, "worse fitted SFS entries"), 
                 cex.names=0.8)
         barplot(rbind(sfs.obs.pol,sfs.exp.pol)[,ordered.indices][,1:numBadEntriesToPLotBIS], col=c("black","grey"), 
                 names.arg = worse.sfs.labels, beside = TRUE, las=2, ylab= "number of sites", cex.lab=1, 
                 main=paste(numBadEntriesToPLotBIS, "worse fitted SFS entries"), 
                 cex.names=0.8)
         legend("top", legend =c("Observed", "Expected"), pch=15,  col=c("black","grey"), cex=0.8)
         par(mfrow=c(1,1))
         dev.off()
      }
      cat("done!\n\n")
      
   }
   
   #===================================================================================================
   #-----------  Extracting lhoods and ML parameter values for each model -----------------------------
   #===================================================================================================
   if (summarizeRresults) {
      cat(paste("Extracting max loglhood and ML parameters for model", curcase), sep="\n")
      
      #Extract maxLhood values
      bestDir=paste(genericName, "-bestRun-OO", numRuns, sep="")
      curbestFile=paste(bestDir, "/", genericName,"/", genericName,".bestlhoods", sep="")  #MODIF Another folder into run/
      curBestParams=read.table(curbestFile, header=T)
      maxEstLhood = curBestParams[["MaxEstLhood"]]
      maxObsLhood = curBestParams[["MaxObsLhood"]]
      
      bestEstLhoods[curcase]=maxEstLhood
      bestObsLhoods[curcase]=maxObsLhood
      numParams=length(curBestParams)
      bestParams[[curcase]]=curBestParams[-c(numParams, numParams-1)]
      
      cat("done!\n\n")
   }
}

if (summarizeRresults) {
   names(bestEstLhoods)=curModels
   names(bestObsLhoods)=curModels
   
   names(bestParams)=curModels
   print(rbind(bestEstLhoods,bestObsLhoods))
   print(bestParams)
   blank.line="\n"
   write.table(blank.line, file=result.file, quote=F, sep="\t", col.names = F, row.names = F)
   write.table(t(c("Lhoods",curModels)), file=result.file, append=T, quote=F, sep="\t", row.names = F, col.names = F)
   write.table(rbind(bestEstLhoods,bestObsLhoods), file=result.file, quote=F, sep="\t", append=T, col.names = F)
   write.table(blank.line, file=result.file, quote=F, sep="\t", append=T, col.names = F, row.names = F)
   for (i in 1:length(curModels)) {
      write.table(c(names(bestParams)[i], bestEstLhoods[i]), file=result.file, quote=F, sep="\t", append=T, row.names = F, col.names = F)
      write.table(bestParams[[i]], file=result.file, quote=F, sep="\t", append=T, row.names = F)
      write.table(blank.line, file=result.file, quote=F, sep="\t", append=T, col.names = F, row.names = F)
   }
}

if ((extractBestRun) & (summarizeRresults)){
    png(paste(result.plot,".png",sep=""), height=3000, width=2000, res=300)
    par(mar=c(9,5,2,1))
    boxplot(MatLhoods,las=1, main="MaxEstLhood", names=NA, cex.axis=1,ylim=c(min(MatLhoods),max(bestObsLhoods)))
    text(seq_along(curModels),y = par("usr")[3], srt = 30, adj = 1,labels = curModels, xpd = TRUE, cex=0.7)
    points(c(1:length(curModels)),bestObsLhoods,pch=8,col="blue")
    text(c(1:length(curModels)),bestObsLhoods, col="blue",labels = bestObsLhoods, cex=0.9, pos=3)
    points(c(1:length(curModels)),bestEstLhoods,pch=8,col="red")
    text(c(1:length(curModels)),bestEstLhoods, col="red",labels = bestEstLhoods, cex=0.9, pos=1)
    dev.off()
    
    png(paste(result.plot,"_DeltaLhoodsObsExp.png",sep=""), height=3000, width=2000, res=300)
    par(mar=c(12,7,2,1))
    boxplot(bestObsLhoods-MatLhoods,las=1, main="Delta Lhoods", names=NA, cex.axis=1,ylim=c(min(bestObsLhoods-MatLhoods),max(bestObsLhoods-MatLhoods)))
    text(seq_along(curModels),y = par("usr")[3], srt = 30, adj = 1,labels = curModels, xpd = TRUE, cex=0.7)
    points(c(1:length(curModels)),bestObsLhoods-bestEstLhoods,pch=8,col="red")
    text(c(1:length(curModels)),(bestObsLhoods-bestEstLhoods), col="red",labels = round(bestObsLhoods-bestEstLhoods,0), cex=0.9, pos=1)
    dev.off()
}